from smtplib import SMTP

from django.core.exceptions import ValidationError
from django.core.mail import EmailMultiAlternatives, get_connection
from django.db import models
from django.template import Context, Template
from django.template.defaultfilters import mark_safe, striptags
from django.template.loader import get_template


class ConnectionProfile(models.Model):
    host = models.CharField(max_length=60)
    username = models.CharField(max_length=40, blank=True)
    password = models.CharField(max_length=40, blank=True)
    port = models.PositiveSmallIntegerField()
    use_tls = models.BooleanField(default=True)

    def __str__(self):
        return '{username} @ {host}:{port}'.format(**vars(self))

    def clean(self):
        try:
            smtp = SMTP()
            smtp.connect(self.host, self.port)
            self.use_tls and smtp.starttls()
            self.username and smtp.login(self.username, self.password)
        except Exception as err:
            raise ValidationError(str(err))


class EmailTemplate(models.Model):
    name = models.SlugField(primary_key=True)
    connection_profile = models.ForeignKey('ConnectionProfile')
    base_template_name = models.CharField(
        max_length=70, blank=True, default='email/default.html')
    subject = models.CharField(max_length=40)
    content = models.TextField()

    @staticmethod
    def get(name):
        return EmailTemplate.objects.get(name=name)

    def __unicode__(self):
        return self.subject

    def clean(self):
        self.name = self.name.upper()

    def render(self, to, context, from_email=''):
        context = Context(dict(context, subject=self.subject))
        subject = Template(self.subject).render(context)
        base_template = get_template(self.base_template_name)
        content = mark_safe(Template(self.content).render(context))
        body = base_template.render(Context({'content': content}))

        connection = get_connection()
        connection.__dict__.update(**dict(
            (field, getattr(self.connection_profile, field))
            for field in ('host', 'port', 'use_tls', 'username', 'password',)))

        # accept a single email as a string
        if not isinstance(to, (list, tuple)):
            to = [to]

        email = EmailMultiAlternatives(
            subject, striptags(body), from_email, to, connection=connection)
        email.attach_alternative(body, 'text/html')

        return email



from django.core import mail
from django.core.exceptions import ValidationError
from django.test import TestCase

from emailer.models import ConnectionProfile, EmailTemplate


class EmailTransportTestCase(TestCase):
    fixtures = ['email_templates']

    def test_connection(self):
        # wrong everything
        conn = ConnectionProfile(
            host='such.an.unknown.n0n3Xizt3ht-server.com',
            port=587, use_tls=False,
            username='foobar', password='123')
        self.assertRaises(ValidationError, lambda: conn.clean())

        # valid server, bad credentials
        conn.__dict__.update(host='smtp.gmail.com', use_tls=True)
        self.assertRaises(ValidationError, lambda: conn.clean())

        # TODO: test a valid connection with a dummy server

    def test_email_send(self):
        to = 'user@server.com'

        # render and send the email from a template
        email = EmailTemplate.get('TEST_EMAIL').render(
            to=to, context={'sender': 'Foo', 'destiny': 'Bar'})
        email.send()

        self.assertEqual(len(mail.outbox), 1)
        sent_email = mail.outbox[0]

        # make sure we're using the right connection
        connection_profile = ConnectionProfile.objects.all()[0]
        for field in ('host', 'port', 'use_tls', 'username', 'password',):
            self.assertEqual(
                getattr(connection_profile, field),
                getattr(sent_email.connection, field))

        # test email content
        self.assertEqual(set(sent_email.to), set([to]))
        self.assertIn('Bar', sent_email.subject)
        self.assertIn('Foo', sent_email.body)



from django import forms
from django.contrib import admin

from emailer.models import ConnectionProfile, EmailTemplate


class ConnectionProfileAdmin(admin.ModelAdmin):

    class form(forms.ModelForm):

        class Meta:
            model = ConnectionProfile
            widgets = {
                'password': forms.PasswordInput,
            }


class EmailTemplateAdmin(admin.ModelAdmin):
    list_display = ('name', 'subject', 'connection_profile',)


admin.site.register(ConnectionProfile, ConnectionProfileAdmin)
admin.site.register(EmailTemplate, EmailTemplateAdmin)



from setuptools import find_packages, setup


setup(
    name='django-emailer',
    version='0.2',
    author='Evandro Myller',
    author_email='emyller@7ws.co',
    description='Template based email sending with SMTP connection management',
    url='https://github.com/7ws/django-emailer',
    install_requires=[
        'django >= 1.5',
    ],
    packages=find_packages(),
    include_package_data=True,
    keywords=['django', 'email', 'smtp'],
    classifiers=[
        'Framework :: Django',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python',
        'Topic :: Software Development',
    ],
)
